// ********************************************************************
// DS1307 - Grundfunktionen 
// AVR-Version / GGC 4.3.2
// Version 1.0 / Stand: 22.12.2009
// Author: Matthias Kahnt
// ********************************************************************
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// der Authoren erforderlich
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// I2C-Basisfunktionen basierend auf:
// Peter Fleury <pfleury@gmx.ch>  http://jump.to/fleury
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************
//
//
//*********************************************************************
// I2C Bus Initialisierung
//*********************************************************************
void i2c_init (void) {
  TWSR = 0;                             // TWPS1=0, TWPS0=0 Prescalar=1
  TWBR = ((F_CPU/SCL_CLOCK)-16)/2;      // Bitrate (muss >10 sein)
}

//*********************************************************************
// I2C Bus Startbedingung
// Rckgabe: 0 - Device vorhanden (ok)
//           1 - Fehler
//*********************************************************************
unsigned char i2c_start (unsigned char i2c_addresse) {
unsigned char twst;
  // send START condition
  TWCR = (1<<TWINT) | (1<<TWSTA) | (1<<TWEN);
  // wait until transmission completed
  while(!(TWCR & (1<<TWINT)));
  // check value of TWI Status Register. Mask prescaler bits.
  twst = TW_STATUS & 0xF8;
  if ((twst != TW_START) && (twst != TW_REP_START)) return 1;
  // send device address
  TWDR = i2c_addresse;
  TWCR = (1<<TWINT) | (1<<TWEN);
  // wail until transmission completed and ACK/NACK has been received
  while(!(TWCR & (1<<TWINT)));
  // check value of TWI Status Register. Mask prescaler bits.
  twst = TW_STATUS & 0xF8;
  if ((twst != TW_MT_SLA_ACK) && (twst != TW_MR_SLA_ACK)) return 1;
  return 0;
}

//*********************************************************************
// I2C Bus
//*********************************************************************
void i2c_start_wait (unsigned char i2c_addresse) {
unsigned char twst;
  while (1) {
    // send START condition
    TWCR = (1<<TWINT) | (1<<TWSTA) | (1<<TWEN);
    // wait until transmission completed
    while(!(TWCR & (1<<TWINT)));
    // check value of TWI Status Register. Mask prescaler bits.
    twst = TW_STATUS & 0xF8;
    if ((twst != TW_START) && (twst != TW_REP_START)) continue;
       // send device address
       TWDR = i2c_addresse;
       TWCR = (1<<TWINT) | (1<<TWEN);
       // wail until transmission completed
       while(!(TWCR & (1<<TWINT)));
       // check value of TWI Status Register. Mask prescaler bits.
       twst = TW_STATUS & 0xF8;
       if ((twst == TW_MT_SLA_NACK )||(twst ==TW_MR_DATA_NACK)) {            
            // device busy, send stop condition to terminate write operation
            TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO);
            // wait until stop condition is executed and bus released
            while(TWCR & (1<<TWSTO));
            continue;
       }
    break;
  }
}

//*********************************************************************
// I2C Bus Startbedingung
// Rckgabe: 0 - Device vorhanden (ok)
//           1 - Fehler
//*********************************************************************
unsigned char i2c_rep_start (unsigned char i2c_addresse) {
  return i2c_start(i2c_addresse);
}

//*********************************************************************
// I2C Bus Stopbedingung
//*********************************************************************
void i2c_stop (void) {
  // send stop condition
  TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWSTO);
  // wait until stop condition is executed and bus released
  while(TWCR & (1<<TWSTO));
}

//*********************************************************************
// I2C Bus Byte senden
// bergabe: Sende-Byte
// Rckgabe: 0 - Device vorhanden (ok)
//           1 - Fehler
//*********************************************************************
unsigned char i2c_write (unsigned char i2c_data) {    
unsigned char twst;
  // send data to the previously addressed device
  TWDR = i2c_data;
  TWCR = (1<<TWINT) | (1<<TWEN);
  // wait until transmission completed
  while(!(TWCR & (1<<TWINT)));
  // check value of TWI Status Register. Mask prescaler bits
  twst = TW_STATUS & 0xF8;
  if( twst != TW_MT_DATA_ACK) return 1;
  return 0;
}

//*********************************************************************
// I2C Bus Byte empfangen mit ACK
// Rckgabe: Empfangs-Byte
//*********************************************************************
unsigned char i2c_readAck (void) {
  TWCR = (1<<TWINT) | (1<<TWEN) | (1<<TWEA);
  while(!(TWCR & (1<<TWINT)));    
  return TWDR;
}

//*********************************************************************
// I2C Bus Byte empfangen mit NAK
// Rckgabe: Empfangs-Byte
//*********************************************************************
unsigned char i2c_readNak (void) {
  TWCR = (1<<TWINT) | (1<<TWEN);
  while(!(TWCR & (1<<TWINT)));
  return TWDR;
}

// ********************************************************************
// BCD zu Binr Konvertierung
// bergabe: bcd_wert - BCD-Wert
// Rckgabe: Binr-Wert
// ********************************************************************
unsigned char bcd_bin (unsigned char bcd_wert) {
  return (bcd_wert >> 4) * 10 + (bcd_wert & 0x0F);
}

// ********************************************************************
// Binr zu BCD Konvertierung
// bergabe: bin_wert - Binr-Wert
// Rckgabe: BCD-Wert
// ********************************************************************
unsigned char bin_bcd (unsigned char bin_wert) {
  return ((bin_wert / 10) << 4) + bin_wert % 10;
}
