// *************************************************************************************
// ASTRO-Schaltuhr/Dmmerungsschalter
// PC-Programm zur Einstellung des Gertes
// Version 1.0 / Stand: 13.01.2010
// *************************************************************************************
// Erstellt mit: Code::Blocks IDE 8.02 inkl. MinGW 32 GNU GCC C-Compiler
// Download unter: http://www.codeblocks.org
// *************************************************************************************
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// des Authors erforderlich.
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// *************************************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// *************************************************************************************

// *************************************************************************************
// Include-Bereich
// *************************************************************************************
#include <stdio.h>      // Standard IO-Funktionen
#include <stdlib.h>     // Standard Bibliothek
#include <string.h>     // Zeichenketten-Funktionen
#include <ctype.h>      // Zeichentest-Funktionen
#include <time.h>       // Zeit-Funktionen (Tutorial unter: www.proggen.org)
#include <afxres.h>     // UART-Funktionen

// *************************************************************************************
// Variablenbereich Programm
// *************************************************************************************
#define MAX_BYTE 1000
char buffer[MAX_BYTE];
unsigned char buffer_index  = 0;          // Index auf eine Stelle im Buffer
unsigned char buffer_anzahl = 0;          // Anzahl der Bytes im Buffer
unsigned char ausgabe_en    = 0;          // Meldungsausgabe am Bildschirm (1=ja 0=nein)
unsigned char receive_en    = 0;          // Empfangsfreigabe (1=ja 0=nein)
unsigned char error         = 0;          // Fehlerstatus 0=kein Fehler
unsigned char eintrag_temp  = 0;          // Angeforderte Eintrags-Nr.
unsigned int  var_temp      = 0;          // Temporre Verwendung
int warten                  = 0;          // Wartezeit in [ms]
char *comport;                            // RS-232 Port Zeiger
unsigned char ps_ist        = 0;          // Prfsumme ist
unsigned char ps_soll       = 0;          // Prfsumme soll
time_t sys_zeit_absolut;                  // Systemzeit in Sekunden seit 1.1.1970 00:00:00 UTC
struct tm *sys_zeit;                      // Zeitstempel Strukturvariable

// *************************************************************************************
// Variablenbereich UART/COM
// *************************************************************************************
BOOL            bPortReady;
DCB             dcb;
COMMTIMEOUTS    CommTimeouts;
BOOL            bWriteRC;
BOOL            bReadRC;
DWORD           iBytesWritten;
DWORD           iBytesRead;
HANDLE          hComPort;

// *************************************************************************************
// Delay-Funktion
// bergabe: Wartezeit in [ms]
// *************************************************************************************
void sleep(int _wait) {
	clock_t goal;
	goal = clock() + _wait;
	while(goal > clock());
}

// *************************************************************************************
// UART/COM: Initialisierung der seriellen Schnittstelle
// Rckgabe: Handle der COM-Verbindung
// *************************************************************************************
HANDLE SerialInit(char *ComPortName, int BaudRate) {
    HANDLE hCom;
    hCom = CreateFile(ComPortName,
        GENERIC_READ | GENERIC_WRITE,
        0,                                                  // exclusive access
        NULL,                                               // no security
        OPEN_EXISTING,
        0,                                                  // no overlapped I/O
        NULL);                                              // null template
    bPortReady              = SetupComm(hCom, 128, 128);    // set buffer sizes 128=Input 128=Output
    bPortReady              = GetCommState(hCom, &dcb);
    dcb.BaudRate            = BaudRate;                     // Baudrate
    dcb.ByteSize            = 8;                            // Datenbits
    dcb.Parity              = NOPARITY;                     // keine Paritt
    dcb.StopBits            = 2;                            // 2 Stopp-Bits
    dcb.fAbortOnError       = TRUE;                         // Abbruch bei Fehlern
    dcb.fOutX               = FALSE;                        // XON/XOFF aus fr senden
    dcb.fInX                = FALSE;                        // XON/XOFF aus fr empfangen
    dcb.fOutxCtsFlow        = FALSE;                        // CTS Flusskontrolle deaktivieren
    dcb.fRtsControl         = FALSE;                        // RTS Flusskontrolle deaktivieren
    dcb.fOutxDsrFlow        = FALSE;                        // DSR Flusskontrolle deaktivieren
    dcb.fDtrControl         = DTR_CONTROL_DISABLE;          // DTR Flusskontrolle deaktivieren
    dcb.fNull               = FALSE;
    dcb.fErrorChar          = FALSE;
    dcb.fDsrSensitivity     = FALSE;
    dcb.fTXContinueOnXoff   = FALSE;
    dcb.wReserved           = 0;                            // muss immer "0" sein!
    bPortReady = SetCommState(hCom, &dcb);
    // Communication timeouts are optional
    bPortReady = GetCommTimeouts (hCom, &CommTimeouts);
    CommTimeouts.ReadIntervalTimeout            = 5000;
    CommTimeouts.ReadTotalTimeoutConstant       = 5000;
    CommTimeouts.ReadTotalTimeoutMultiplier     = 1000;
    CommTimeouts.WriteTotalTimeoutConstant      = 5000;
    CommTimeouts.WriteTotalTimeoutMultiplier    = 1000;
    bPortReady = SetCommTimeouts (hCom, &CommTimeouts);
    return hCom;
}

// *************************************************************************************
// UART/COM: Datenblock-Empfangsroutine in den Buffer
// Rckgabe: 0=ok 1=Fehler
// *************************************************************************************
char ReceiveFromASU() {
    int index;
    BOOL bReadRC;
    DWORD iBytesRead;
    sleep(10);
    for(index = 0; index < buffer_anzahl; index++) {
        bReadRC = ReadFile(hComPort,&buffer[index],1,&iBytesRead,NULL);
        if (!iBytesRead) return 1;    // Fehler, zB. keine Verbindung
        if (ausgabe_en) printf("*%02d ",buffer[index]);
        sleep(10);
    }
    return 0;
}

// *************************************************************************************
// UART/COM: Senden des Buffer-Inhaltes
// *************************************************************************************
void SendToASU() {
    int index;
    BOOL bWriteRC;
    DWORD iBytesWritten;
    for(index = 0; index < buffer_anzahl; index++) {
        bWriteRC = WriteFile(hComPort,&buffer[index],1,&iBytesWritten,NULL);
        if (ausgabe_en) printf("%2d ",buffer[index]);
        sleep(10);
    }
}

// *************************************************************************************
// Berechnung der Prfsumme ber den Buffer-Inhalt
// Rckgabe: Prfsumme
// *************************************************************************************
unsigned char CheckSummeBuffer(void) {
    unsigned char temp = 0;
    for(buffer_index = 0; buffer_index < (buffer_anzahl - 1); buffer_index++) {
        temp += buffer[buffer_index];
    }
    return temp;
}

// *************************************************************************************
// Hauptprogramm
// *************************************************************************************
int main(int argc, char* argv[]) {
    if (argc < 3) {
        printf("-> Fehler: zu wenig Argumente!\n");
        return -1;
    }
    if ((tolower(*argv[1]) != 'c') || (tolower(*(argv[1] + 1)) != 'o') || (tolower(*(argv[1] + 2)) != 'm')) {
        printf("-> Fehler: %s keine serielle Schnittstelle!\n",argv[1]);
        return -1;
    }
    comport = argv[1]; // COM-Port
    // Funktion selektieren
    switch(*argv[2]) {
        case 'M':   // Manuellbetrieb
        case 'm':   // Format: asu COM1 M K1=1 (K1=1 K1=0 K2=1 K2=0)
                    // Format zur ASU: 'M' 10 01 K1 K2 00 00 00 00 PS
                    if ((argc < 4) || (strlen(argv[3]) != 4)) {
                        printf("-> Fehler: M-Formatfehler (Bsp.: COM1 M K2=1)!\n");
                        return -1;
                    }
                    buffer_anzahl = 10; // Bufferbelegung
                    buffer[0] = 'M';    // Kennbyte
                    buffer[1] = 10;     // Blocklnge 10 Byte
                    buffer[2] = 1;      // Festwert, immer 1
                    switch (*(argv[3] + 1)) {
                        case '1':   // Relais K1
                                    if (*(argv[3] + 3) == '1') buffer[3] = 1;   // K1 Ein
                                        else buffer[3] = 0;                     // K1 Aus
                                    buffer[4] = 2;                              // K2 wird nicht beeinflusst
                                    break;
                        case '2':   // Relais K2
                                    buffer[3] = 2;                              // K1 wird nicht beeinflusst
                                    if (*(argv[3] + 3) == '1') buffer[4] = 1;   // K2 Ein
                                         else buffer[4] = 0;                    // K2 Aus
                                    break;
                        default:    // Fehler
                                    error = 1;
                    }
                    buffer[5] = 0;                  // Reserviert fr Erweiterungen
                    buffer[6] = 0;                  // Reserviert fr Erweiterungen
                    buffer[7] = 0;                  // Reserviert fr Erweiterungen
                    buffer[8] = 0;                  // Reserviert fr Erweiterungen
                    buffer[9] = CheckSummeBuffer(); // Prfsumme
                    receive_en = 1;                 // Empfang freigeben
                    break;
        case 'P':   // Einstellungen
        case 'p':   // Format: asu COM1 P D=0 S=100 (D Dmmerungsart=0,1,2 S Standortkennzahl=0 ..122)
                    // Format zur ASU: 'P' 10 01 D S 00 00 00 00 PS
                    if (argc < 5) {
                        printf("-> Fehler: P-Formatfehler (Bsp.: COM1 P D=0 S=55)!\n");
                        return -1;
                    }
                    buffer_anzahl = 10; // Bufferbelegung
                    buffer[0] = 'P';    // Kennbyte
                    buffer[1] = 10;     // Blocklnge 10 Byte
                    buffer[2] = 1;      // Festwert, immer 1
                    buffer[3] = 0;
                    switch (*(argv[3] + 2)) {
                        case '0':   buffer[3] = 0; break;   // bgerliche Dmmerung
                        case '1':   buffer[3] = 1; break;   // nautische Dmmerung
                        case '2':   buffer[3] = 2; break;   // astronomische Dmmerung
                        default:    error = 1;              // Fehler
                    }
                    var_temp = atoi(argv[4] + 2);          // Standortkennzahl
                    if (var_temp > 122) error = 1;    // Fehler
                    buffer[4] = (unsigned char) var_temp;
                    buffer[5] = 0;                  // Reserviert fr Erweiterungen
                    buffer[6] = 0;                  // Reserviert fr Erweiterungen
                    buffer[7] = 0;                  // Reserviert fr Erweiterungen
                    buffer[8] = 0;                  // Reserviert fr Erweiterungen
                    buffer[9] = CheckSummeBuffer(); // Prfsumme
                    receive_en = 1;                 // Empfang freigeben
                    warten = 1000;                  // Wartezeit am Ende des Programms
                    break;
        case 'Z':   // Uhrzeit/Datum setzen (Es wird die PC-Systemzeit zur Astroschaltuhr bertragen)
        case 'z':   // Da die Ermittlung des Status der MESZ/MEZ unzuverlssig ist, muss der aktuelle
                    // Stand in der Kmdo.-Zeile mit angegeben werden!
                    // Format: asu COM1 Z MESZ=0 (0=Normalzeit 1=Sommerzeit)
                    if ((argc < 4) || (strlen(argv[3]) != 6)) {
                        printf("-> Fehler: Z-Formatfehler (Bsp.: COM1 Z MESZ=1)!\n");
                        return -1;
                    }
                    time(&sys_zeit_absolut);                    // Systemzeit in Sekunden seit 1.1.1970 00:00:00 UTC
                    sys_zeit = localtime (&sys_zeit_absolut);   // Zeitstempel in Struktur bernehmen
                    buffer_anzahl = 12;     // Bufferbelegung
                    buffer[0]  = 'Z';       // Kennbyte
                    buffer[1]  = 12;        // Blocklnge 10 Byte
                    buffer[2]  = 1;         // Festwert, immer 1
                    buffer[3]  = sys_zeit->tm_mday;             // Tag
                    buffer[4]  = sys_zeit->tm_mon + 1;          // Monat
                    buffer[5]  = sys_zeit->tm_year - 100;       // Jahr
                    buffer[6]  = sys_zeit->tm_wday;             // Wochentag
                    if (!buffer[6]) buffer[6] = 7;              // Umkodierung des Sonntag = 7
                    buffer[7]  = sys_zeit->tm_hour;             // Stunde
                    buffer[8]  = sys_zeit->tm_min;              // Minute
                    buffer[9]  = sys_zeit->tm_sec;              // Sekunde
                    buffer[10] = 0;
                    if (*(argv[3] + 5) == '1') buffer[10] = 1;  // Sommerzeit MESZ ggf. setzen
                    buffer[11] = CheckSummeBuffer();            // Prfsumme
                    receive_en = 1;                             // Empfang freigeben
                    warten = 1200;                              // Wartezeit am Ende des Programms
                    break;
        case 'R':   // Stautsbericht aus der Astroschaltuhr auslesen
        case 'r':   // Format: asu COM1 R
                    buffer_anzahl = 1;  // Bufferbelegung
                    buffer[0]  = 'R';   // Kennbyte
                    receive_en = 1;     // Empfang freigeben
                    break;
        case 'D':   // Schaltprogramm-Eintrag aus der Astroschaltuhr auslesen
        case 'd':   // Format: asu COM1 D 1 (Eintrags-Nr. 1 ... 20)
                    // Format zur ASU: 'D' 04 NR PS
                    if (argc < 4) {
                        printf("-> Fehler: D-Formatfehler (Bsp.: COM1 D 10)!\n");
                        return -1;
                    }
                    buffer_anzahl = 4;  // Bufferbelegung
                    buffer[0] = 'D';    // Kennbyte
                    buffer[1] = 4;      // Blocklnge 4 Byte
                    buffer[2] = (unsigned char) atoi(argv[3]);  // Eintrags-Nr.
                    buffer[3] = CheckSummeBuffer();             // Prfsumme
                    if ((buffer[2] > 20) || (buffer[2] < 1)) error = 1; // Fehler
                    eintrag_temp = buffer[2];                   // Eintrags-Nr. merken
                    receive_en   = 1;                           // Empfang freigeben
                    break;
        case 'E':   // Gesamtes Schaltprogramm im EEPROM speichern
        case 'e':   // Format: asu COM1 E
                    buffer_anzahl = 1;  // Bufferbelegung
                    buffer[0]  = 'E';   // Kennbyte
                    receive_en = 1;     // Empfang freigeben
                    warten     = 2000;  // Wartezeit am Ende des Programms
                    break;
        case 'S':   // Schaltprogramm-Eintrag zur Astroschaltuhr senden
        case 's':   // Format: asu COM1 S 10 "++--++- SA + 00:00 SU - 01:58 K1 E"
                    // Beachten! Formatierung exakt einhalten
                    // Gesamtblock immer in Anfhrungszeichen einschliessen
                    // Eintrags-Nr. immer zweistellig mit fhrender Null, also 01,02,03 ... 19,20
                    // Wochentage Mo,Di,Mi,Do,Fr,Sa,So +=aktiv, -=inaktiv
                    // SA=Sonnenaufgang, SU=Sonnenuntergang, MD=Morgendmmerung, AD=Abenddmmerung, FZ=Festzeit
                    // Zeiten immer vierstellig, also 01:30, 00:10 usw.
                    // K1, K2 und K3 zulssig (K3 schaltet beide Relais gleichzeitig ein)
                    // E/D Eintrag freigeben/sperren
                    // Format zur ASU: 'S' 16 NR WT EF ES EM EV AF AS AM AV K1 K2 EN PS
                    if (argc < 5) {
                        printf("-> Fehler: S-Formatfehler (Bsp.: COM1 S 10 \x22...\x22)\n");
                        return -1;
                    }
                    if (strlen(argv[4]) != 34) {
                        printf("-> Fehler: S-Formatfehler im Eintragsstring!\n");
                        return -1;
                    }
                    buffer_anzahl = 16; // Bufferbelegung
                    buffer[0] = 'S';    // Kennbyte
                    buffer[1] = 16;     // Blocklnge 16 Byte
                    buffer[2] = (unsigned char) atoi(argv[3]);  // Eintrags-Nr.
                    if ((buffer[2] > 20) || (buffer[2] < 1)) error = 1;
                    // Wochentage +++++-- (Mo,Di,Mi,Do,Fr,Sa,So +=Aktiv)
                    buffer[3] = 0;
                    if (*argv[4] == '+') buffer[3] |= 0x02;     // Montag
                    argv[4]++;
                    if (*argv[4] == '+') buffer[3] |= 0x04;     // Dienstag
                    argv[4]++;
                    if (*argv[4] == '+') buffer[3] |= 0x08;     // Mittwoch
                    argv[4]++;
                    if (*argv[4] == '+') buffer[3] |= 0x10;     // Donnerstag
                    argv[4]++;
                    if (*argv[4] == '+') buffer[3] |= 0x20;     // Freitag
                    argv[4]++;
                    if (*argv[4] == '+') buffer[3] |= 0x40;     // Samstag
                    argv[4]++;
                    if (*argv[4] == '+') buffer[3] |= 0x01;     // Sonntag
                    argv[4]++; argv[4]++;
                    // Einschaltfunktion (SA SU MD AD FZ)
                    buffer[4] = 5;  // Nur zur Fehlerkontrolle
                    if ((*argv[4] == 'S') && (*(argv[4] + 1) == 'A')) buffer[4] = 0;     // Sonnenaufgang
                    if ((*argv[4] == 'S') && (*(argv[4] + 1) == 'U')) buffer[4] = 2;     // Sonnenuntergang
                    if ((*argv[4] == 'M') && (*(argv[4] + 1) == 'D')) buffer[4] = 1;     // Morgendmmerung
                    if ((*argv[4] == 'A') && (*(argv[4] + 1) == 'D')) buffer[4] = 3;     // Abenddmmerung
                    if ((*argv[4] == 'F') && (*(argv[4] + 1) == 'Z')) buffer[4] = 4;     // Festzeit
                    if (buffer[4] > 4) error = 1;
                    argv[4]++; argv[4]++; argv[4]++;
                    // Vorzeichen des Offsets der Einschaltfunktion
                    buffer[7] = 0;                              // Vorzeichen positiv
                    if (*argv[4] == '-') buffer[7] = 1;         // Vorzeichen negativ
                    argv[4]++; argv[4]++;
                    // Offset bzw. Einschaltzeit (Stunde)
                    buffer[5] = (char) atoi(argv[4]);  // Stunde
                    if (buffer[4] < 4) {
                        // Sonnenstandsabhngige Funktionen
                        if (buffer[5] > 1) error = 1;           // hchstens 1h erlaubt
                    } else {
                        // Festzeit
                        if (buffer[5] > 23) error = 1;          // 0 .. 23 erlaubt
                    }
                    argv[4]++; argv[4]++; argv[4]++;
                    // Offset bzw. Einschaltzeit (Minute)
                    buffer[6] = (char) atoi(argv[4]);  // Minute
                    if (buffer[6] > 59) error = 1;              // hchstens 59Min erlaubt
                    argv[4]++; argv[4]++; argv[4]++;
                    // Ausschaltfunktion (SA SU MD AD FZ)
                    buffer[8] = 5;  // Nur zur Fehlerkontrolle
                    if ((*argv[4] == 'S') && (*(argv[4] + 1) == 'A')) buffer[8] = 0;     // Sonnenaufgang
                    if ((*argv[4] == 'S') && (*(argv[4] + 1) == 'U')) buffer[8] = 2;     // Sonnenuntergang
                    if ((*argv[4] == 'M') && (*(argv[4] + 1) == 'D')) buffer[8] = 1;     // Morgendmmerung
                    if ((*argv[4] == 'A') && (*(argv[4] + 1) == 'D')) buffer[8] = 3;     // Abenddmmerung
                    if ((*argv[4] == 'F') && (*(argv[4] + 1) == 'Z')) buffer[8] = 4;     // Festzeit
                    if (buffer[8] > 4) error = 1;
                    argv[4]++; argv[4]++; argv[4]++;
                    // Vorzeichen des Offsets der Einschaltfunktion
                    buffer[11] = 0;                             // Vorzeichen positiv
                    if (*argv[4] == '-') buffer[11] = 1;        // Vorzeichen negativ
                    argv[4]++; argv[4]++;
                    // Offset bzw. Einschaltzeit (Stunde)
                    buffer[9] = (char) atoi(argv[4]);  // Stunde
                    if (buffer[8] < 4) {
                        // Sonnenstandsabhngige Funktionen
                        if (buffer[9] > 1) error = 1;           // hchstens 1h erlaubt
                    } else {
                        // Festzeit
                        if (buffer[9] > 23) error = 1;          // 0 .. 23 erlaubt
                    }
                    argv[4]++; argv[4]++; argv[4]++;
                    // Offset bzw. Einschaltzeit (Minute)
                    buffer[10] = (char) atoi(argv[4]);  // Minute
                    if (buffer[10] > 59) error = 1;   // hchstens 59Min erlaubt
                    argv[4]++; argv[4]++; argv[4]++; argv[4]++;
                    // Relais
                    buffer[12] = 0;
                    buffer[13] = 0;
                    if (*argv[4] == '1') buffer[12] = 1;    // K1 Ein
                    if (*argv[4] == '2') buffer[13] = 1;    // K2 Ein
                    if (*argv[4] == '3') {
                        buffer[12] = 1;    // K1 Ein
                        buffer[13] = 1;    // K2 Ein
                    }
                    argv[4]++; argv[4]++;
                    buffer[14] = 0;
                    if (*argv[4] == 'E') buffer[14] = 1;    // Eintrag aktiv
                    buffer[15] = CheckSummeBuffer();        // Prfsumme
                    receive_en = 1;                         // Empfang freigeben
                    break;
        default:    // Fehler
                    error = 1;
    }
    if (error) {
        // Analysefehler
        printf("-> Fehler: Befehlsformat nicht korrekt!\n");
        return -1;
    }
    // COM-Port ffnen
	hComPort = SerialInit(comport, 9600); // 9600 Baud 8N1, keine Flusskontrolle
	if( hComPort == (HANDLE) -1) {
	    // Portfehler
	    printf("-> Fehler: Initialisierung Port: %s\n",comport);
		return -1;
    }
    // Buffer seriell senden ber RS232 zur Astroschaltuhr
    SendToASU();
    // Buffer seriell empfangen ber RS232 von der Astroschaltuhr falls erforderlich
    if (receive_en) {
        switch (*argv[2]) {
            case 'M':   // Besttigung des Manuellbetriebes empfangen und auswerten
            case 'm':   buffer_anzahl = 1;          // Bufferbelegung (Anzahl der zu empfangenden Bytes)
                        error = ReceiveFromASU();   // Empfangen
                        if (error) {
                            // Empfangsfehler/Timeout
                            printf("-> Fehler: Empfangsfehler/Timeout Port: %s\n",comport);
                            CloseHandle(hComPort);
                            return -1;
                        }
                        if (buffer[0] != 'M') {
                            // Fehler
                            printf("-> Fehler: Kommando nicht bestaetigt!\n");
                            CloseHandle(hComPort);
                            return -1;
                        }
                        break;
            case 'P':   // Besttigung der Einstellungen empfangen und auswerten
            case 'p':   buffer_anzahl = 1;          // Bufferbelegung (Anzahl der zu empfangenden Bytes)
                        error = ReceiveFromASU();   // Empfangen
                        if (error) {
                            // Empfangsfehler/Timeout
                            printf("-> Fehler: Empfangsfehler/Timeout Port: %s\n",comport);
                            CloseHandle(hComPort);
                            return -1;
                        }
                        if (buffer[0] != 'P') {
                            // Fehler
                            printf("-> Fehler: Kommando nicht bestaetigt!\n");
                            CloseHandle(hComPort);
                            return -1;
                        }
                        break;
            case 'Z':   // Besttigung des Uhrzeit/Datum stellen empfangen und auswerten
            case 'z':   buffer_anzahl = 1;          // Bufferbelegung (Anzahl der zu empfangenden Bytes)
                        error = ReceiveFromASU();   // Empfangen
                        if (error) {
                            // Empfangsfehler/Timeout
                            printf("-> Fehler: Empfangsfehler/Timeout Port: %s\n",comport);
                            CloseHandle(hComPort);
                            return -1;
                        }
                        if (buffer[0] != 'Z') {
                            // Fehler
                            printf("-> Fehler: Kommando nicht bestaetigt!\n");
                            CloseHandle(hComPort);
                            return -1;
                        }
                        break;
            case 'S':   // Besttigung des Schaltprogramm-Eintrages empfangen und auswerten
            case 's':   buffer_anzahl = 1;          // Bufferbelegung (Anzahl der zu empfangenden Bytes)
                        error = ReceiveFromASU();   // Empfangen
                        if (error) {
                            // Empfangsfehler/Timeout
                            printf("-> Fehler: Empfangsfehler/Timeout Port: %s\n",comport);
                            CloseHandle(hComPort);
                            return -1;
                        }
                        if (buffer[0] != 'S') {
                            // Fehler
                            printf("-> Fehler: Kommando nicht bestaetigt!\n");
                            CloseHandle(hComPort);
                            return -1;
                        }
                        break;
            case 'E':   // Besttigung des EEPROM Schreibvorganges empfangen und auswerten
            case 'e':   buffer_anzahl = 1;          // Bufferbelegung (Anzahl der zu empfangenden Bytes)
                        error = ReceiveFromASU();   // Empfangen
                        if (error) {
                            // Empfangsfehler/Timeout
                            printf("-> Fehler: Empfangsfehler/Timeout Port: %s\n",comport);
                            CloseHandle(hComPort);
                            return -1;
                        }
                        break;
            case 'R':   // Statusbericht empfangen und auswerten
            case 'r':   buffer_anzahl = 31;          // Bufferbelegung (Anzahl der zu empfangenden Bytes)
                        error = ReceiveFromASU();    // Empfangen
                        if (error) {
                            // Empfangsfehler/Timeout
                            printf("-> Fehler: Empfangsfehler/Timeout Port: %s\n",comport);
                            CloseHandle(hComPort);
                            return -1;
                        }
                        // Statusbericht liegt vor, Prfsumme checken
                        ps_soll = buffer[30];           // Bufferbelegung
                        ps_ist  = CheckSummeBuffer();   // Prfsumme
                        if (ps_ist != ps_soll) {
                            // Prfsumme falsch
                            printf("-> Fehler: Pruefsumme falsch!\n");
                            CloseHandle(hComPort);
                            return -1;
                        }
                        // Prfsumme stimmt, Statusbericht ausgeben
                        printf("\nStatusbericht:\n--------------\n");
                        printf("Datum: %02d.%02d.20%02d, ",buffer[3],buffer[4],buffer[5]);
                        switch(buffer[6]) {
                            case 1: printf("Montag"); break;
                            case 2: printf("Dienstag"); break;
                            case 3: printf("Mittwoch"); break;
                            case 4: printf("Donnerstag"); break;
                            case 5: printf("Freitag"); break;
                            case 6: printf("Samstag"); break;
                            case 7: printf("Sonntag"); break;
                        }
                        printf("\nZeit:  %02d:%02d:%02d Uhr ",buffer[7],buffer[8],buffer[9]);
                        switch(buffer[10]) {
                            case 0: printf("(MEZ-Normalzeit)\n");  break;
                            case 1: printf("(MESZ-Sommerzeit)\n"); break;
                        }
                        printf("Morgendaemmerung:    %02d:%02d Uhr\n",buffer[11],buffer[12]);
                        printf("Sonnenaufgang:       %02d:%02d Uhr\n",buffer[16],buffer[17]);
                        if (buffer[15] < 0) printf("Sonnenhoechststand:  11:%02d Uhr\n",60 + buffer[15]);
                            else            printf("Sonnenhoechststand:  12:%02d Uhr\n",buffer[15]);
                        printf("Sonnenuntergang:     %02d:%02d Uhr\n",buffer[18],buffer[19]);
                        printf("Abenddaemmerung:     %02d:%02d Uhr\n",buffer[13],buffer[14]);
                        printf("Standort-Kennzahl:   %03d\n",buffer[20]);
                        printf("geografische Breite: %02d.%02d Grad\n",buffer[21],buffer[22]);
                        printf("geografische Laenge: %02d.%02d Grad\n",buffer[23],buffer[24]);
                        printf("Daemmerungsart:      ");
                        switch(buffer[25]) {
                            case 0: printf("0 - buergerliche Daemmerung -6 Grad\n"); break;
                            case 1: printf("1 - nautische Daemmerung -12 Grad\n"); break;
                            case 2: printf("2 - astronomische Daemmerung -18 Grad\n"); break;
                        }
                        printf("Relaiszustaende:     ");
                        switch(buffer[26]) {
                            case 0: printf("K1: Aus, "); break;
                            case 1: printf("K1: Ein, "); break;
                        }
                        switch(buffer[27]) {
                            case 0: printf("K2: Aus\n"); break;
                            case 1: printf("K2: Ein\n"); break;
                        }
                        printf("Firmware-Version:    %02d\n",buffer[28]);
                        printf("Systemstatus:        ");
                        if (buffer[29] > 0) printf("Fehler-Nr. %02d\n",buffer[29]);
                            else            printf("Ok\n");
                        break;
            case 'D':   // Schaltprogramm-Eintrag empfangen und auswerten
            case 'd':   buffer_anzahl = 16;         // Bufferbelegung (Anzahl der zu empfangenden Bytes)
                        error = ReceiveFromASU();   // Empfangen
                        if (error) {
                            // Empfangsfehler/Timeout
                            printf("-> Fehler: Empfangsfehler/Timeout Port: %s\n",comport);
                            CloseHandle(hComPort);
                            return -1;
                        }
                        // Schaltprogramm-Eintrag liegt vor, Prfsumme checken
                        ps_soll = buffer[15];           // Bufferbelegung
                        ps_ist  = CheckSummeBuffer();   // Prfsumme
                        if (ps_ist != ps_soll) {
                            // Prfsumme falsch
                            printf("-> Fehler: Pruefsumme falsch!\n");
                            CloseHandle(hComPort);
                            return -1;
                        }
                        // Prfsumme stimmt
                        //printf("Schaltprogramm:\n---------------\n");
                        printf("Eintrag-Nr.: %02d  ",eintrag_temp);
                        // Wochentage
                        if (buffer[3] & 0x02) printf("+"); else printf("-");  // Montag
                        if (buffer[3] & 0x04) printf("+"); else printf("-");  // Dienstag
                        if (buffer[3] & 0x08) printf("+"); else printf("-");  // Mittwoch
                        if (buffer[3] & 0x10) printf("+"); else printf("-");  // Donnerstag
                        if (buffer[3] & 0x20) printf("+"); else printf("-");  // Freitag
                        if (buffer[3] & 0x40) printf("+"); else printf("-");  // Samstag
                        if (buffer[3] & 0x01) printf("+"); else printf("-");  // Sonntag
                        // EIN-Funktion
                        printf(" EIN: ");
                        switch(buffer[4]) {
                            case 0: printf("SA"); break;    // Sonnenaufgang
                            case 1: printf("MD"); break;    // Morgendmmerung
                            case 2: printf("SU"); break;    // Sonnenuntergang
                            case 3: printf("AD"); break;    // Abenddmmerung
                            case 4: printf("FZ"); break;    // Feste Zeit
                        }
                        // Vorzeichen
                        if (buffer[4] != 4) {
                            // Sonnenstandsabhnge Funktionen
                            if (buffer[5] || buffer[6]) {
                                // Offsetwert vorhanden
                                if (!buffer[7]) printf("+"); else printf("-");
                                printf("%02d:%02dh",buffer[5],buffer[6]);
                            }
                        } else {
                            // Festzeit
                            printf(" %02d:%02d Uhr",buffer[5],buffer[6]);
                        }
                        // AUS-Funktion
                        printf(" AUS: ");
                        switch(buffer[8]) {
                            case 0: printf("SA"); break;    // Sonnenaufgang
                            case 1: printf("MD"); break;    // Morgendmmerung
                            case 2: printf("SU"); break;    // Sonnenuntergang
                            case 3: printf("AD"); break;    // Abenddmmerung
                            case 4: printf("FZ"); break;    // Feste Zeit
                        }
                        // Vorzeichen
                        if (buffer[8] != 4) {
                            // Sonnenstandsabhnge Funktionen
                            if (buffer[9] || buffer[10]) {
                                // Offsetwert vorhanden
                                if (!buffer[11]) printf("+"); else printf("-");
                                printf("%02d:%02dh",buffer[9],buffer[10]);
                            }
                        } else {
                            // Festzeit
                            printf(" %02d:%02d Uhr",buffer[9],buffer[10]);
                        }
                        // Schaltaktivitten
                        if (buffer[12]) printf(" K1=EIN");
                        if (buffer[13]) printf(" K2=EIN");
                        // Enablestatus
                        if (buffer[14]) printf(" (Eintrag aktiv)\n"); else printf(" (Eintrag nicht aktiv)\n");
                        break;

        }
    }
    // COM-Port schliessen
    CloseHandle(hComPort);
    if (warten) sleep(warten);  // Wartezeit bei Notwendigkeit
    printf(" -> ok\n");
    return 0;
}
